/*
 * Copyright (c) 2007, 2008 by Hartmut Birr
 *
 * This program is free software; you can redistribute it and/or
 * mmodify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA. 
 *
 */

#ifndef __DCG_H__
#define __DCG_H__

#include <inttypes.h>
#include <avr/eeprom.h>

#define	DC2mA       0
#define DC20mA      1
#define DC200mA     2
#define DC2000mA    3

typedef struct
{
    uint8_t RelaisState;

    uint8_t PwrInRange      : 1;

    uint8_t LCDPresent      : 1;

} FLAGS;

typedef union
{
    uint8_t u8;
    struct
    {
        uint8_t Unused      : 1;
        uint8_t FuseBlown   : 1;
        uint8_t OverVolt    : 1;
        uint8_t OverTemp    : 1;
        uint8_t EEUnlocked  : 1;
        uint8_t CurrentMode : 1;
        uint8_t UserSRQ     : 1;
        uint8_t Busy        : 1;
    };
} STATUS;

typedef struct
{
    uint8_t ADC16Present : 1;
    uint8_t DAC16Present : 1;
} OPTIONS;

typedef struct
{
    int16_t DACUOffsets[2];
    double DACUScales[2];
    int16_t DACIOffsets[4];
    double DACIScales[4];
    int16_t ADCUOffsets[2];
    double ADCUScales[2];
    int16_t ADCIOffsets[4];
    double ADCIScales[4];
    double InitVoltage;
    double InitCurrent;
    double GainPre;
    double GainOut;
    double GainI;
    double RefVoltage;
    double MaxVoltage[2];
    double RSense[4];
    double MaxCurrent[4];
    double ADCUfacs[2];
    OPTIONS Options;
    double RelayVoltage;
    double FanOnTemp;
    uint8_t IncRast;
    uint8_t SerBaudReg;
    uint8_t TrackChSave;
    double GainPwrIn;
    uint8_t LockRangeU;
    uint8_t LockRangeI;
    uint16_t Initialised;
} PARAMS;

extern FLAGS Flags;
extern STATUS Status;
extern PARAMS Params;
extern PARAMS eepParams EEMEM;

extern double DCVoltMod;
extern double DCAmpMod;
extern double Temperature;
extern double xVoltage;
extern double xCurrent;

extern double wVoltage;
extern double wCurrent;

extern uint16_t ADCRawU;
extern uint16_t ADCRawI;
extern uint16_t DACRawU;
extern uint16_t DACRawI;

extern uint8_t SlaveCh;
extern uint8_t TrackCh;
extern uint16_t ErrCount;

extern uint8_t RangeI;
extern uint8_t RangeU;

extern uint8_t LockRangeI;
extern uint8_t LockRangeU;


extern const char VersStrLong[];
extern const char SerInpStr[];


// dcg.c //////////////////////////////////////////////////
double GetPowerIn(void);
void InitScales(void);
void SetLevelDAC(void);
void CheckLimits(void);
uint8_t CalcRangeI(double);
void SetActivityTimer(uint8_t);


// dcg-hw.c ///////////////////////////////////////////////
uint16_t GetADC(uint8_t);
#ifdef DUAL_DAC 
void ShiftOut1655(uint16_t, uint8_t);
#else
void ShiftOut1655(uint16_t);
#endif
void ShiftOut1257(uint16_t);
uint16_t ShiftIn1864(void);
double GetLM75Temp(void);
void ConfigLM75(double);

// dcg-panel.h ////////////////////////////////////////////
void jobPanel(void);

// dcg-parser.h ///////////////////////////////////////////
void SendTrackCmd(void);

#endif
