/*
 * Copyright (c) 2007 by Hartmut Birr
 *
 * This program is free software; you can redistribute it and/or
 * mmodify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301, USA.
 *
 */

#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

#ifndef ENCODER_PORT_A
#define ENCODER_PORT_A	PINA
#endif

#ifndef ENCODER_PORT_B
#define ENCODER_PORT_B	PINA
#endif

#ifndef ENCODER_PIN_A
#define ENCODER_PIN_A	PA0
#endif

#ifndef ENCODER_PIN_B
#define ENCODER_PIN_B	PA1
#endif

int8_t EncPos;
uint8_t EncState;

void jobEncoder(void)
{
    uint8_t next = EncState;
    uint8_t i;
    uint8_t cntA, cntB;

    for (i = cntA = cntB = 0; i < 5; i++)
    {
        _delay_us(2);
        if (ENCODER_PORT_A & (1<<ENCODER_PIN_A))
        {
            cntA++;
        }
        if (ENCODER_PORT_B & (1<<ENCODER_PIN_B))
        {
            cntB++;
        }
    }

    if (next & 0x01)
    {
        if (cntA < 2)
        {
            next &= ~0x01;
        }
    }
    else
    {
        if (cntA > 3)
        {
            next |= 0x01;
        }
    }

    if (next & 0x02)
    {
        if (cntB < 2)
        {
            next &= ~0x02;
        }
    }
    else
    {
        if (cntB > 3)
        {
            next |= 0x02;
        }
    }


    if ((EncState == 1 && next == 3) ||
        (EncState == 3 && next == 2) ||
        (EncState == 2 && next == 0) ||
        (EncState == 0 && next == 1))
    {
        EncPos--;
    }
    else if ((EncState == 2 && next == 3) ||
             (EncState == 3 && next == 1) ||
             (EncState == 1 && next == 0) ||
             (EncState == 0 && next == 2))
    {
        EncPos++;
    }

    EncState = next;
}

int8_t GetEncoderPos(uint8_t reset)
{
    uint8_t sreg;
    int8_t pos;

    sreg = SREG;
    cli();

    pos = EncPos;

    if (reset)
    {
        EncPos = 0;
    }

    SREG = sreg;

    return pos;
}

int8_t GetAndResetEncPos(void)
{
    return GetEncoderPos(1);
}
